const shell = require('shelljs')
const path = require('path')
const fs = require('fs')
const rimraf = require('rimraf')
const getHelp = require('./help.js')
const envDefine = require('./build-en-define.json')
const sha256 = require('./sha256')
const clc = require('./clc')

const customs = fs.readdirSync(path.resolve(process.cwd(), 'src/custom'))
let all = customs.reduce((result, dir) => {
  // ignore directories whose name starts without `apaas-custom-plugin-`
  if (dir.indexOf('xdap-plugin-')) return result
  try {
    const apaasJson = JSON.parse(fs.readFileSync(path.resolve(process.cwd(), 'src', 'custom', dir, 'apaas.json')))
    // match every plugins except those whose local.notAll is true
    if (apaasJson && (!apaasJson.local || !apaasJson.local.notAll)) result.push(dir.slice(20))
  } catch (e) {
  } finally {
    return result
  }
}, [])

const help = getHelp(all)
let crypto = 'noc'
let noZip = 'noz'
let batchMode = false
let plugin = ''
let async = false
let env = envDefine.join(',')
let outputSha256 = false

const pickValue = function(a, i) {
  return a && a[i] && !a[i].startsWith('-') && a[i]
}

const parseArgv = function(argv) {
  const localArgv = [...argv]
  for (let i = 0; i < localArgv.length; i++) {
    if (localArgv[i].startsWith('--')) {
      switch (localArgv[i]) {
        case '--clean':
        case '--rm':
          rimraf.sync(path.resolve(process.cwd(), 'crypto'))
          rimraf.sync(path.resolve(process.cwd(), 'crypto*.zip'))
          rimraf.sync(path.resolve(process.cwd(), 'PLUGIN_*'))
          rimraf.sync(path.resolve(process.cwd(), 'PLUGIN_*.zip'))
          process.exit(0)
        case '--help':
          console.log(help)
          process.exit(0)
        default:
          break
      }
    } else if (localArgv[i].startsWith('-')) {
      const flag = localArgv[i].slice(0)
      const flags = (flag && flag.split('')) || []
      flags.forEach((f) => {
        let a
        switch (f) {
          case 'c':
            crypto = '-c'
            break
          case 'b':
            batchMode = true
            break
          case 'a':
            async = true
            break
          case 'A':
            ;(a = pickValue(localArgv, i + 1)) && a && ((all = a.split(',')), i++)
            break
          case 'Z':
            noZip = '-Z'
            break
          case 'i':
            ;(a = pickValue(localArgv, i + 1)) && a && ((plugin = a), i++)
            break
          case 'h':
            console.log(help)
            process.exit(0)
          case 'e':
            ;(a = pickValue(localArgv, i + 1)) && a && ((env = a), i++)
          case 's':
            outputSha256 = true
          default:
            break
        }
      })
    }
  }
  if (!batchMode && !plugin) {
    val = process.argv.slice(2, 3)[0]
    val && !val.startsWith('-') && (plugin = val)
    if (!plugin) {
      console.error('未指定插件名')
      process.exit(0)
    }
  }
}

parseArgv(process.argv.slice(2))

const scriptPath = path.resolve(process.cwd(), 'build/build-plugins.js')

const doBuild = (item) => {
  shell.exec(`node ${scriptPath} ${item} ${crypto} ${noZip} ${env} ${outputSha256 ? 'sha' : ''}`, { async: async })
}

if (outputSha256) {
  sha256.create()
}

if (batchMode) {
  all.forEach(doBuild)
} else {
  doBuild(plugin)
}

sha256.show()
sha256.close()
