const parseHtml = function(sourceCode) {
  const element = document.createElement('div')
  // eslint-disable-next-line no-irregular-whitespace
  element.innerHTML = sourceCode.replace(/﻿/gi, '')
  return element.childNodes
}

const stringifyDom = function(dom) {
  const element = document.createElement('div')
  // [...dom]是因为appendChild会从arrayDom中删除该节点，导致forEach遍历提前结束
  const arrayDom = dom.length ? [...dom] : [dom]
  arrayDom.forEach((ele) => {
    element.appendChild(ele)
  })
  return element.innerHTML
}

const traverseDom = function(dom, traverseFun) {
  if (dom && traverseFun instanceof Function) {
    const arrayDom = dom.length ? dom : [dom]
    const stack = [...arrayDom]
    while (stack.length) {
      const current = stack.pop()
      if (current.childNodes && current.childNodes.length) {
        stack.push(...current.childNodes)
      }
      traverseFun(current, stack)
    }
  }
  return dom
}

const replaceQlClassByDom = function(dom) {
  const classMap = {
    'ql-align-right': 'text-align:right;',
    'ql-align-center': 'text-align:center;',
    'ql-align-justify': 'text-align:justify;',
    'ql-size-huge': 'font-size:18px;',
    'ql-size-large': 'font-size:16px;',
    'ql-size-small': 'font-size:12px;',
    'ql-indent-1': 'padding-left:3em;',
    'ql-indent-2': 'padding-left:6em;',
    'ql-indent-3': 'padding-left:9em;',
    'ql-indent-4': 'padding-left:12em;',
    'ql-indent-5': 'padding-left:15em;',
    'ql-indent-6': 'padding-left:18em;',
    'ql-indent-7': 'padding-left:21em;',
    'ql-indent-8': 'padding-left:24em;'
  }

  traverseDom(dom, (node) => {
    if (node.classList && node.classList.length) {
      // text节点无classList
      const classList = [...node.classList]
      let style = ''
      classList.forEach((item) => {
        if (item && classMap[item]) {
          style += classMap[item]
        }
      })
      if (style) {
        node.style.cssText += style
      }
    }
  })
  return dom
}

const replaceQlClass = function(sourceCode) {
  const dom = parseHtml(sourceCode)
  const tmp1 = replaceQlClassByDom(dom)
  return stringifyDom(tmp1)
}

const replaceFormFieldByDom = function(dom) {
  traverseDom(dom, (node) => {
    const classList = node.classList
    if (classList) {
      const mention = [...classList].includes('mention')
      if (mention) {
        const dataset = node.dataset
        if (dataset && dataset.id) {
          node.innerText = '${_' + dataset.id + '}'
        }
      }
    }
  })
  return dom
}

const replaceFormField = function(sourceCode) {
  const dom = parseHtml(sourceCode)
  const tmp1 = replaceFormFieldByDom(dom)
  return stringifyDom(tmp1)
}

const cutLabel = function(source) {
  // eslint-disable-next-line no-template-curly-in-string
  return source.replace(/\$\{_([\w\d]+?)_.*?\}/gi, '${_$1}')
}

const parseAll = function(sourceCode) {
  const dom = parseHtml(cutLabel(sourceCode))
  const tmp = replaceQlClassByDom(dom)
  const tmp2 = replaceFormFieldByDom(tmp)
  return stringifyDom(tmp2)
}

export default { replaceQlClass, replaceFormField, parseAll, stringifyDom, cutLabel }
