// migrated by Mosuzi

import Vue from 'vue'
import VueDOMPurifyHTML from 'vue-dompurify-html'
import GlobalEventDirective from '../directives/global-event.directive'
import Base64 from 'js-base64'

const initVueDOMPurifyHTML = () => {
  Vue.use(VueDOMPurifyHTML, {
    hooks: {
      afterSanitizeAttributes: (currentNode) => {
        // Do something with the node
        // set all elements owning target to target=_blank
        if ('target' in currentNode) {
          currentNode.setAttribute('target', '_blank')
          // prevent https://www.owasp.org/index.php/Reverse_Tabnabbing
          currentNode.setAttribute('rel', 'noopener noreferrer')
        }
      }
    }
  })
}

const initLoadScript = () => {
  const LoadScriptPlugin = {
    install: function(Vue) {
      const load = async function(src, attrObject, beforeBody = false) {
        // eslint-disable-line no-param-reassign
        return new Promise(function(resolve, reject) {
          if (document.querySelector('script[src="' + src + '"]')) {
            resolve()
            return
          }

          const el = document.createElement('script')

          el.type = 'text/javascript'
          el.async = true
          el.src = src
          if (Object.prototype.toString.call(attrObject) === '[object Object]') {
            Object.keys(attrObject).forEach((key) => {
              el.setAttribute(key, attrObject[key])
            })
          }

          el.addEventListener('load', resolve)
          el.addEventListener('error', reject)
          el.addEventListener('abort', reject)
          if (beforeBody) {
            document.body.appendChild(el)
          } else {
            document.head.appendChild(el)
          }
        })
      }

      const blockLoadAll = async function(scriptsArray) {
        return new Promise(function(resolve, reject) {
          const promise = scriptsArray.reduce((promise, current) => {
            return promise
              .then(() => {
                return Vue.loadScript(current.script, current.attrObject, current.beforeBody)
              })
              .catch(() => {
                return Vue.loadScript(current.script, current.attrObject, current.beforeBody)
              })
          }, Vue.loadScript(scriptsArray[0].script, scriptsArray[0].attrObject, scriptsArray[0].beforeBody))
          promise
            .then(() => {
              resolve()
            })
            .catch((err) => {
              reject(err)
            })
        })
      }

      const unload = async function(src, beforeBody) {
        // eslint-disable-line no-param-reassign
        return new Promise(function(resolve, reject) {
          const el = document.querySelector('script[src="' + src + '"]')

          if (!el) {
            // reject(new Error('未加载指定的脚本'))
            // return
          } else {
            if (beforeBody) {
              document.body.removeChild(el)
            } else {
              document.head.removeChild(el)
            }
          }

          resolve()
        })
      }

      Vue._loadScript = Vue.prototype.$loadScript = {
        load: load,
        unload: unload,
        blockLoadAll: blockLoadAll
      }
    }
  }

  Vue.use(LoadScriptPlugin)
}

const initLoadCss = () => {
  const LoadCssPlugin = {
    install: function(Vue) {
      const load = async function(src, attrObject, beforeBody = false) {
        // eslint-disable-line no-param-reassign
        return new Promise(function(resolve, reject) {
          if (document.querySelector('link[href="' + src + '"]')) {
            resolve()
            return
          }

          const el = document.createElement('link')
          el.href = src
          el.type = 'text/css'
          el.rel = 'stylesheet'
          if (Object.prototype.toString.call(attrObject) === '[object Object]') {
            Object.keys(attrObject).forEach((key) => {
              el.setAttribute(key, attrObject[key])
            })
          }

          el.addEventListener('load', resolve)
          el.addEventListener('error', reject)
          el.addEventListener('abort', reject)
          if (beforeBody) {
            document.body.appendChild(el)
          } else {
            document.head.appendChild(el)
          }
        })
      }

      const unload = async function(src, beforeBody) {
        // eslint-disable-line no-param-reassign
        return new Promise(function(resolve, reject) {
          const el = document.querySelector('link[href="' + src + '"]')

          if (!el) {
            // reject(new Error('未加载指定的CSS脚本'))
            // return
          } else {
            if (beforeBody) {
              document.body.removeChild(el)
            } else {
              document.head.removeChild(el)
            }
          }

          resolve()
        })
      }

      Vue._loadCss = Vue.prototype.$loadCss = {
        load: load,
        unload: unload
      }
    }
  }

  Vue.use(LoadCssPlugin)
}

initVueDOMPurifyHTML()

initLoadScript()
initLoadCss()

Vue.prototype.$base64 = Base64.Base64

Vue.directive('global-event', GlobalEventDirective)
