/*
 * @Author: lizhihang
 * @Date: 2020-12-24 10:48:41
 * @LastEditors: lizhihang
 * @LastEditTime: 2020-12-28 11:47:49
 * @Description: 选人弹窗配置
 * @FilePath: \x-project-app\src\vendor\form-modal-config\person-modal.config.js
 */
import Vue from 'vue'
import apis from '@/api'

const loadUserData = (page, pageSize, keyWord) => {
  return new Promise((resolve, reject) => {
    const request = {
      ...apis.QUERY_ALL_USERS,
      params: {
        page,
        pageSize,
        keyWord
      }
    }
    Vue.prototype
      .$request(request)
      .asyncThen(
        (resp) => {
          if (resp.code === 'ok') {
            resolve({
              data: resp.data,
              total: resp.total
            })
          }
        },
        (err) => {
          reject(err)
        }
      )
      .asyncErrorCatch((err) => {
        reject(err)
      })
  })
}

const loadDataFilter = (event) => {
  const { page, pageSize, keyWord } = event
  return new Promise((resolve, reject) => {
    const request = {
      ...apis.QUERY_ADMIN_USER_LIST,
      params: {
        page,
        pageSize,
        keyWord
      }
    }
    Vue.prototype
      .$request(request)
      .asyncThen(
        (resp) => {
          if (resp.code === 'ok') {
            resolve({
              data: resp.data,
              total: resp.total
            })
          }
        },
        (err) => {
          reject(err)
        }
      )
      .asyncErrorCatch((err) => {
        reject(err)
      })
  })
}

const loadDepartmentData = (deptId, keyword) => {
  return new Promise((resolve, reject) => {
    const request = {
      ...apis.QUERY_DEPARTMENT_INFO,
      params: {
        deptId: deptId || '',
        keyword: keyword || ''
      }
    }
    Vue.prototype
      .$request(request)
      .asyncThen(
        (resp) => {
          if (resp.code === 'ok') {
            resolve({
              data: resp.data,
              total: resp.total
            })
          }
        },
        (err) => {
          reject(err)
        }
      )
      .asyncErrorCatch((err) => {
        reject(err)
      })
  })
}

const loadDeptUserData = (page, pageSize, deptId, includeSubdepartments) => {
  return new Promise((resolve, reject) => {
    const request = {
      ...apis.QUERY_DEPARTMENT_USERS,
      params: {
        deptId,
        page,
        pageSize,
        includeSubdepartments
      }
    }
    Vue.prototype
      .$request(request)
      .asyncThen(
        (resp) => {
          if (resp.code === 'ok') {
            resolve({
              data: resp.data,
              total: resp.total
            })
          }
        },
        (err) => {
          reject(err)
        }
      )
      .asyncErrorCatch((err) => {
        reject(err)
      })
  })
}

const loadRoleData = (appId) => {
  return new Promise((resolve, reject) => {
    const request = {
      ...apis.QUERY_ROLE_INFO,
      params: {}
    }
    if (appId) {
      request.params = {
        appId
      }
    }
    Vue.prototype
      .$request(request)
      .asyncThen(
        (resp) => {
          if (resp.code === 'ok') {
            resolve({
              data: resp.data,
              total: resp.total
            })
          }
        },
        (err) => {
          reject(err)
        }
      )
      .asyncErrorCatch((err) => {
        reject(err)
      })
  })
}

const loadRoleUserData = (page, pageSize, roleId) => {
  return new Promise((resolve, reject) => {
    const request = {
      ...apis.QUERY_ROLE_USERS,
      params: {
        page,
        pageSize,
        roleId
      }
    }
    Vue.prototype
      .$request(request)
      .asyncThen(
        (resp) => {
          if (resp.code === 'ok') {
            resolve({
              data: resp.data,
              total: resp.total
            })
          }
        },
        (err) => {
          reject(err)
        }
      )
      .asyncErrorCatch((err) => {
        reject(err)
      })
  })
}

const loadUserDataFilter = (param, formId, uuid, formDetailData, currentRowTableData) => {
  return new Promise((resolve, reject) => {
    const request = {
      ...apis.QUERY_ALL_USERS_FILTER,
      params: {
        formId,
        componentId: uuid,
        param,
        formData: formDetailData,
        currentRowTableData: currentRowTableData
      }
    }
    Vue.prototype
      .$request(request)
      .asyncThen(
        (resp) => {
          if (resp.code === 'ok') {
            resolve({
              data: resp.data,
              total: resp.total
            })
          }
        },
        (err) => {
          reject(err)
        }
      )
      .asyncErrorCatch((err) => {
        reject(err)
      })
  })
}

export default {
  loadUserData: loadUserData,
  loadDataFilter: loadDataFilter,
  loadDepartmentData: loadDepartmentData,
  loadDeptUserData: loadDeptUserData,

  loadRoleData: loadRoleData,
  loadRoleUserData: loadRoleUserData,
  loadUserDataFilter: loadUserDataFilter
}
